/******************************************************************************
 *            echecs_coups_jouables.c                                         *
 *                                                                            *
 *  Copyright  2006  gerald.dumas                                             *
 *  linechec@laposte.net                                                      *
 ******************************************************************************
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  any later version.                                                        *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

/* fonction de recherche des cases jouables pour chaque piece	*/
/* Calcul aussi le nombre de cases jouables par couleur	*/

#include "echecs_coups_jouables.h"

GList *
ajout_coup_dans_liste (GList * liste_coups_jouables, gint couleur,
		       gint position_piece_tab, gint coup,
		       gboolean case_occupee, gboolean coup_ami)
{
  struct struct_coups_jouables *ncoup = NULL;

  ncoup =
    (struct struct_coups_jouables *)
    g_malloc (sizeof (struct struct_coups_jouables));
  if (ncoup == NULL)
    {
      printf
	(" pas assez de mémoire pour créer un coup jouable supplémentaire.\n");
      gtk_main_quit ();
    }
  ncoup->coup = coup;
  ncoup->couleur = couleur;
  ncoup->pos_piece_tab = position_piece_tab;
  ncoup->case_occupee = case_occupee;
  ncoup->coup_ami = coup_ami;
  ncoup->nbre_coups_adverse = 0;
  liste_coups_jouables = g_list_append (liste_coups_jouables, ncoup);
  return liste_coups_jouables;
}

gboolean
case_attaquee_par_couleur (gint8 case_a_tester, gint8 couleur)
{
  gint8 couleur_locale;
  gint8 position_roi;
  gboolean resultat = FALSE;
  type_piece piece_originale;
  struct struct_echiquier case_originale_depart;
  struct struct_echiquier case_originale_arrivee;

  if (couleur == blanche)
    {
      position_roi = 32;
      couleur_locale = noire;
    }
  else
    {
      position_roi = 16;
      couleur_locale = blanche;
    }

  // Sauvegarde des données
  case_originale_depart = echiquier[table_piece[position_roi].place];
  piece_originale = table_piece[position_roi];
  case_originale_arrivee = echiquier[case_a_tester];

  // effacement de la piece sur l'échiquier
  echiquier[table_piece[position_roi].place].type_piece = vide;

  // Initialisation de la nouvelle position dans table_piece
  table_piece[position_roi].place = case_a_tester;

  // positionnement de la pièce sur l'échiquier
  echiquier[table_piece[position_roi].place].type_piece = roi;
  echiquier[table_piece[position_roi].place].couleur_piece = couleur_locale;
  echiquier[table_piece[position_roi].place].pos_table_piece = position_roi;

  if (le_roi_est_en_echec (couleur_locale))
    resultat = TRUE;

  // restauration des données originales avec MAJ du nbre de coups jouables
  table_piece[position_roi] = piece_originale;
  echiquier[table_piece[position_roi].place] = case_originale_depart;
  echiquier[case_a_tester] = case_originale_arrivee;

  return resultat;
}

type_piece piece;

GList *
cases_jouables (GList * liste_coups_jouables, gint pos_table)
{
  gint couleur_piece;
  gint locale = 0;
  register gint x;

  // détemination du type de la pièce
  couleur_piece = piece.couleur;

  //rechercher de quelle pièce il s'agit et traîtement.
  switch (piece.type)
    {
    case pion:
      {
	if (couleur_piece == noire)
	  {
	    piece.nbre_cases_jouables = 0;

	    // si la case est vide le coup est jouable
	    if (echiquier[piece.place + 12].type_piece == vide)
	      {
		liste_coups_jouables =
		  ajout_coup_dans_liste (liste_coups_jouables, noir,
					 pos_table, piece.place + 12, FALSE,
					 FALSE);
		piece.nbre_cases_jouables++;
	      }

	    if (piece.deplacement_initial)	// 1er déplacement du pion.
	      {
		if ((echiquier[piece.place + 24].type_piece == vide)
		    && (echiquier[piece.place + 12].type_piece == vide))
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables, noir,
					     pos_table, piece.place + 24,
					     FALSE, FALSE);
		    piece.nbre_cases_jouables++;
		  }
	      }

	    // si la case en biais est occupée par une piece blanche le coup est jouable.
	    if ((echiquier[piece.place + 11].type_piece != bord)
		&& (echiquier[piece.place + 11].type_piece != vide))
	      {
		if (echiquier[piece.place + 11].couleur_piece == blanche)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables, noir,
					     pos_table, piece.place + 11,
					     TRUE, FALSE);
		    piece.nbre_cases_jouables++;
		  }
		else
		  liste_coups_jouables =
		    ajout_coup_dans_liste (liste_coups_jouables, noir,
					   pos_table, piece.place + 11, TRUE,
					   TRUE);
	      }

	    if ((echiquier[piece.place + 13].type_piece != bord)
		&& (echiquier[piece.place + 13].type_piece != vide))
	      {
		if (echiquier[piece.place + 13].couleur_piece == blanche)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables, noir,
					     pos_table, piece.place + 13,
					     TRUE, FALSE);
		    piece.nbre_cases_jouables++;
		  }
		else
		  liste_coups_jouables =
		    ajout_coup_dans_liste (liste_coups_jouables, noir,
					   pos_table, piece.place + 13, TRUE,
					   TRUE);
	      }

	    // Gestion de la prise en passant
	    if (num_coup_joue > 1)	// si au moins un coup à été joué alors ...
	      {
		if ((echiquier[piece.place + 13].type_piece == vide)
		    && (echiquier[piece.place].y == 4)
		    && (echiquier[piece.place + 1].type_piece == pion)
		    && (echiquier[piece.place + 1].couleur_piece == blanche))
		  {
		    if (tab_sauve_partie[num_coup_joue - 1].
			table_piece[echiquier[piece.place + 1].
				    pos_table_piece].deplacement_initial ==
			TRUE)
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables, noir,
						 pos_table, piece.place + 13,
						 FALSE, FALSE);
			piece.nbre_cases_jouables++;
			prise_en_passant = TRUE;
		      }
		  }

		if ((echiquier[piece.place + 11].type_piece == vide)
		    && (echiquier[piece.place].y == 4)
		    && (echiquier[piece.place - 1].type_piece == pion)
		    && (echiquier[piece.place - 1].couleur_piece == blanche))
		  {
		    if (tab_sauve_partie[num_coup_joue - 1].
			table_piece[echiquier[piece.place - 1].
				    pos_table_piece].deplacement_initial ==
			TRUE)
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables, noir,
						 pos_table, piece.place + 11,
						 FALSE, FALSE);
			piece.nbre_cases_jouables++;
			prise_en_passant = TRUE;
		      }
		  }
	      }

	    return liste_coups_jouables;
	    break;
	  }
	else
	  {
	    piece.nbre_cases_jouables = 0;

	    // si la case est vide le coup est jouable
	    if (echiquier[piece.place - 12].type_piece == vide)
	      {
		liste_coups_jouables =
		  ajout_coup_dans_liste (liste_coups_jouables, blanc,
					 pos_table, piece.place - 12, FALSE,
					 FALSE);
		piece.nbre_cases_jouables++;
	      }

	    if (piece.deplacement_initial == TRUE)	// 1er déplacement du pion.
	      {
		if ((echiquier[piece.place - 24].type_piece == vide)
		    && (echiquier[piece.place - 12].type_piece == vide))
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables, blanc,
					     pos_table, piece.place - 24,
					     FALSE, FALSE);
		    piece.nbre_cases_jouables++;
		  }
	      }

	    // si la case en biais est occupée par une piece noire le coup est jouable.
	    if ((echiquier[piece.place - 11].type_piece != bord)
		&& (echiquier[piece.place - 11].type_piece != vide))
	      {
		if (echiquier[piece.place - 11].couleur_piece == noire)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables, blanc,
					     pos_table, piece.place - 11,
					     TRUE, FALSE);
		    piece.nbre_cases_jouables++;
		  }
		else
		  liste_coups_jouables =
		    ajout_coup_dans_liste (liste_coups_jouables, blanc,
					   pos_table, piece.place - 11, TRUE,
					   TRUE);
	      }

	    if ((echiquier[piece.place - 13].type_piece != bord)
		&& (echiquier[piece.place - 13].type_piece != vide))
	      {
		if (echiquier[piece.place - 13].couleur_piece == noire)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables, blanc,
					     pos_table, piece.place - 13,
					     TRUE, FALSE);
		    piece.nbre_cases_jouables++;
		  }
		else
		  liste_coups_jouables =
		    ajout_coup_dans_liste (liste_coups_jouables, blanc,
					   pos_table, piece.place - 13, TRUE,
					   TRUE);
	      }

	    // Gestion de la prise en passant
	    if (num_coup_joue > 1)	// si au moins un coup à été joué alors ...
	      {
		if ((echiquier[piece.place - 13].type_piece == vide)
		    && (echiquier[piece.place].y == 5)
		    && (echiquier[piece.place - 1].type_piece == pion)
		    && (echiquier[piece.place - 1].couleur_piece == noire))
		  {
		    if (tab_sauve_partie[num_coup_joue - 1].
			table_piece[echiquier[piece.place - 1].
				    pos_table_piece].deplacement_initial ==
			TRUE)
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables, blanc,
						 pos_table, piece.place - 13,
						 FALSE, FALSE);
			piece.nbre_cases_jouables++;
			prise_en_passant = TRUE;
		      }
		  }


		if ((echiquier[piece.place - 11].type_piece == vide)
		    && (echiquier[piece.place].y == 5)
		    && (echiquier[piece.place + 1].type_piece == pion)
		    && (echiquier[piece.place + 1].couleur_piece == noire))
		  {
		    if (tab_sauve_partie[num_coup_joue - 1].
			table_piece[echiquier[piece.place + 1].
				    pos_table_piece].deplacement_initial ==
			TRUE)
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables, blanc,
						 pos_table, piece.place - 11,
						 FALSE, FALSE);
			piece.nbre_cases_jouables++;
			prise_en_passant = TRUE;
		      }
		  }
	      }

	    return liste_coups_jouables;
	    break;
	  }
	break;
      }

    case tour:
      {
	if (piece.couleur == noire)
	  couleur_piece = blanche;
	else
	  couleur_piece = noire;

	piece.nbre_cases_jouables = 0;

	for (x = 0; x < 10; x++)
	  {
	    if (rosace[4][x] != 0)	// la rosace n'est pas finie
	      {
		locale = piece.place;
		while (echiquier[locale + rosace[4][x]].type_piece == vide)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables,
					     piece.couleur, pos_table,
					     locale + rosace[4][x], FALSE,
					     FALSE);
		    locale = locale + rosace[4][x];
		    piece.nbre_cases_jouables++;
		  }
		if (echiquier[locale + rosace[4][x]].type_piece != bord)
		  {
		    if (echiquier[locale + rosace[4][x]].couleur_piece ==
			couleur_piece)
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables,
						 piece.couleur, pos_table,
						 locale + rosace[4][x], TRUE,
						 FALSE);
			piece.nbre_cases_jouables++;
		      }
		    else
		      liste_coups_jouables =
			ajout_coup_dans_liste (liste_coups_jouables,
					       piece.couleur, pos_table,
					       locale + rosace[4][x], TRUE,
					       TRUE);
		  }
	      }
	    else
	      x = 10;		// la rosace est finie
	  }

	return liste_coups_jouables;
	break;
      }

    case cavalier:
      {
	if (piece.couleur == noire)
	  couleur_piece = blanche;
	else
	  couleur_piece = noire;

	piece.nbre_cases_jouables = 0;

	for (x = 0; x < 10; x++)
	  {
	    if (rosace[2][x] != 0)	// la rosace n'est pas finie
	      {
		if (echiquier[piece.place + rosace[2][x]].type_piece == vide)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables,
					     piece.couleur, pos_table,
					     piece.place + rosace[2][x],
					     FALSE, FALSE);
		    piece.nbre_cases_jouables++;
		  }
		else
		  {
		    if (echiquier[piece.place + rosace[2][x]].type_piece !=
			bord)
		      {
			if (echiquier[piece.place + rosace[2][x]].
			    couleur_piece == couleur_piece)
			  {
			    liste_coups_jouables =
			      ajout_coup_dans_liste (liste_coups_jouables,
						     piece.couleur, pos_table,
						     piece.place +
						     rosace[2][x], TRUE,
						     FALSE);
			    piece.nbre_cases_jouables++;
			  }
			else
			  liste_coups_jouables =
			    ajout_coup_dans_liste (liste_coups_jouables,
						   piece.couleur, pos_table,
						   piece.place + rosace[2][x],
						   TRUE, TRUE);
		      }
		  }
	      }
	    else
	      x = 10;		// la rosace est finie
	  }
	return liste_coups_jouables;
	break;
      }

    case fou:
      {
	if (piece.couleur == noire)
	  couleur_piece = blanche;
	else
	  couleur_piece = noire;

	piece.nbre_cases_jouables = 0;

	for (x = 0; x < 10; x++)
	  {
	    if (rosace[5][x] != 0)	// la rosace n'est pas finie
	      {
		locale = piece.place;
		while (echiquier[locale + rosace[5][x]].type_piece == vide)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables,
					     piece.couleur, pos_table,
					     locale + rosace[5][x], FALSE,
					     FALSE);
		    locale = locale + rosace[5][x];
		    piece.nbre_cases_jouables++;
		  }
		if (echiquier[locale + rosace[5][x]].type_piece != bord)
		  {
		    if (echiquier[locale + rosace[5][x]].couleur_piece ==
			couleur_piece)
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables,
						 piece.couleur, pos_table,
						 locale + rosace[5][x], TRUE,
						 FALSE);
			piece.nbre_cases_jouables++;
		      }
		    else
		      liste_coups_jouables =
			ajout_coup_dans_liste (liste_coups_jouables,
					       piece.couleur, pos_table,
					       locale + rosace[5][x], TRUE,
					       TRUE);
		  }
	      }
	    else
	      x = 10;		// la rosace est finie
	  }
	return liste_coups_jouables;
	break;
      }

    case dame:
      {
	if (piece.couleur == noire)
	  couleur_piece = blanche;
	else
	  couleur_piece = noire;

	piece.nbre_cases_jouables = 0;

	for (x = 0; x < 10; x++)
	  {
	    if (rosace[6][x] != 0)	// la rosace n'est pas finie
	      {
		locale = piece.place;
		while (echiquier[locale + rosace[6][x]].type_piece == vide)
		  {
		    liste_coups_jouables =
		      ajout_coup_dans_liste (liste_coups_jouables,
					     piece.couleur, pos_table,
					     locale + rosace[6][x], FALSE,
					     FALSE);
		    locale = locale + rosace[6][x];
		    piece.nbre_cases_jouables++;
		  }
		if (echiquier[locale + rosace[6][x]].type_piece != bord)
		  {
		    if (echiquier[locale + rosace[6][x]].couleur_piece ==
			couleur_piece)
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables,
						 piece.couleur, pos_table,
						 locale + rosace[6][x], TRUE,
						 FALSE);
			piece.nbre_cases_jouables++;
		      }
		    else
		      liste_coups_jouables =
			ajout_coup_dans_liste (liste_coups_jouables,
					       piece.couleur, pos_table,
					       locale + rosace[6][x], TRUE,
					       TRUE);
		  }
	      }
	    else
	      x = 10;		// la rosace est finie
	  }
	return liste_coups_jouables;
	break;
      }

    case roi:
      {
	if (couleur_piece == noire)
	  {
	    locale = table_piece[32].place;	// sauvegarde de la position du roi actuelle
	    piece.nbre_cases_jouables = 0;

	    x = 0;
	    while (rosace[3][x] != 0)
	      {
		if (echiquier[piece.place + rosace[3][x]].type_piece == vide)
		  {
		    if (!case_attaquee_par_couleur
			(piece.place + rosace[3][x], blanche))
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables, noir,
						 pos_table,
						 piece.place + rosace[3][x],
						 FALSE, FALSE);
			piece.nbre_cases_jouables++;
		      }
		  }
		else
		  if (echiquier[piece.place + rosace[3][x]].type_piece !=
		      bord)
		  {
		    if (echiquier[piece.place + rosace[3][x]].couleur_piece ==
			blanche)
		      {
			if (!case_attaquee_par_couleur
			    (piece.place + rosace[3][x], blanche))
			  {
			    liste_coups_jouables =
			      ajout_coup_dans_liste (liste_coups_jouables,
						     noir, pos_table,
						     piece.place +
						     rosace[3][x], TRUE,
						     FALSE);
			    piece.nbre_cases_jouables++;
			  }
		      }
		    else
		      liste_coups_jouables =
			ajout_coup_dans_liste (liste_coups_jouables, noir,
					       pos_table,
					       piece.place + rosace[3][x],
					       TRUE, TRUE);
		  }
		x++;
	      }

	    // Etude du petit roque.
	    if (echiquier[31].type_piece == vide
		&& echiquier[32].type_piece == vide
		&& !case_attaquee_par_couleur (31, blanche)
		&& !case_attaquee_par_couleur (32, blanche)
		&& !le_roi_est_en_echec (noir)
		&& table_piece[32].deplacement_initial
		&& table_piece[26].deplacement_initial)
	      {
		liste_coups_jouables =
		  ajout_coup_dans_liste (liste_coups_jouables, noir,
					 pos_table, piece.place + 2, FALSE,
					 FALSE);
		piece.nbre_cases_jouables++;
	      }

	    // Etude du grand roque.
	    if (echiquier[27].type_piece == vide
		&& echiquier[28].type_piece == vide
		&& echiquier[29].type_piece == vide
		&& !case_attaquee_par_couleur (27, blanche)
		&& !case_attaquee_par_couleur (28, blanche)
		&& !case_attaquee_par_couleur (29, blanche)
		&& !le_roi_est_en_echec (noir)
		&& table_piece[32].deplacement_initial
		&& table_piece[25].deplacement_initial)
	      {
		liste_coups_jouables =
		  ajout_coup_dans_liste (liste_coups_jouables, noir,
					 pos_table, piece.place - 2, FALSE,
					 FALSE);
		piece.nbre_cases_jouables++;
	      }

	    return liste_coups_jouables;
	    break;

	  }
	else
	  {
	    locale = table_piece[16].place;	// sauvegarde de la position du roi actuelle
	    piece.nbre_cases_jouables = 0;
	    // Pour chaque case jouable on vérifie si la case est vide, ou si la case
	    // est occupée par une pièce adverse et si le roi adverse ne l'attaque pas
	    // ( Interdiction pour les rois de se mettre mutuellement en echec ).

	    x = 0;
	    while (rosace[3][x] != 0)
	      {
		if (echiquier[piece.place + rosace[3][x]].type_piece == vide)
		  {
		    if (!case_attaquee_par_couleur
			(piece.place + rosace[3][x], noire))
		      {
			liste_coups_jouables =
			  ajout_coup_dans_liste (liste_coups_jouables, blanc,
						 pos_table,
						 piece.place + rosace[3][x],
						 FALSE, FALSE);
			piece.nbre_cases_jouables++;
		      }
		  }
		else
		  if (echiquier[piece.place + rosace[3][x]].type_piece !=
		      bord)
		  {
		    if (echiquier[piece.place + rosace[3][x]].couleur_piece ==
			noire)
		      {
			if (!case_attaquee_par_couleur
			    (piece.place + rosace[3][x], noire))
			  {
			    liste_coups_jouables =
			      ajout_coup_dans_liste (liste_coups_jouables,
						     blanc, pos_table,
						     piece.place +
						     rosace[3][x], TRUE,
						     FALSE);
			    piece.nbre_cases_jouables++;
			  }
		      }
		    else
		      liste_coups_jouables =
			ajout_coup_dans_liste (liste_coups_jouables, blanc,
					       pos_table,
					       piece.place + rosace[3][x],
					       TRUE, TRUE);
		  }
		x++;
	      }

	    // Etude du petit roque.
	    if (echiquier[115].type_piece == vide
		&& echiquier[116].type_piece == vide
		&& !case_attaquee_par_couleur (115, noire)
		&& !case_attaquee_par_couleur (116, noire)
		&& !le_roi_est_en_echec (blanc)
		&& table_piece[16].deplacement_initial
		&& table_piece[10].deplacement_initial)
	      {
		liste_coups_jouables =
		  ajout_coup_dans_liste (liste_coups_jouables, blanc,
					 pos_table, piece.place + 2, FALSE,
					 FALSE);
		piece.nbre_cases_jouables++;
	      }

	    // Etude du grand roque.
	    if (echiquier[111].type_piece == vide
		&& echiquier[112].type_piece == vide
		&& echiquier[113].type_piece == vide
		&& !case_attaquee_par_couleur (111, noire)
		&& !case_attaquee_par_couleur (112, noire)
		&& !case_attaquee_par_couleur (113, noire)
		&& !le_roi_est_en_echec (blanc)
		&& table_piece[16].deplacement_initial
		&& table_piece[9].deplacement_initial)
	      {
		liste_coups_jouables =
		  ajout_coup_dans_liste (liste_coups_jouables, blanc,
					 pos_table, piece.place - 2, FALSE,
					 FALSE);
		piece.nbre_cases_jouables++;
	      }

	    return liste_coups_jouables;
	    break;
	  }
	break;
      }
    }
  return liste_coups_jouables;	// Inutile mais évite une erreur du compilateur
}

GList *
calcul_cases_jouables (GList * liste_coups_jouables)
{
  register gint x;

  // calcul des cases jouables et initialisation des variables de l'échiquier 
  // pour les cases attaquees.
  for (x = 1; x < 32; x++)
    {
      if (x != 16)
	if (table_piece[x].type != vide)
	  {
	    piece = table_piece[x];
	    liste_coups_jouables = cases_jouables (liste_coups_jouables, x);
	    table_piece[x] = piece;
	  }
    }
  piece = table_piece[16];
  liste_coups_jouables = cases_jouables (liste_coups_jouables, 16);
  table_piece[16] = piece;
  piece = table_piece[32];
  liste_coups_jouables = cases_jouables (liste_coups_jouables, 32);
  table_piece[32] = piece;
  return liste_coups_jouables;
}
