/******************************************************************************
 *            echecs_gnuchess.c                                               *
 *                                                                            *
 *  Copyright  2006  gerald.dumas                                             *
 *  linechec@laposte.net                                                      *
 ******************************************************************************
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  any later version.                                                        *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

/* Fonctions pour communiquer avec Gnuchess */
#include "echecs_gnuchess.h"
#include "g_menu.h"

GtkWidget *g_fenetre_attente(GtkWidget *window, gchar *invite, gboolean start)
{
	GtkWidget *box=NULL;
	GtkWidget *barre=NULL;
	static guint idle=0;
	if (start)
	{
		GtkWidget *fenetre_principale;
		fenetre_principale=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "fenetre_principale"));
		// Création d'une fenêtre d'attente le temps que gnuchess se lance
		window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_transient_for(GTK_WINDOW(window),
																								GTK_WINDOW(fenetre_principale));
		gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
		gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
		box=gtk_vbox_new(TRUE, 10);
		gtk_container_add(GTK_CONTAINER(window), box);
		gtk_box_pack_start_defaults(GTK_BOX(box), gtk_label_new(invite));
		barre=gtk_progress_bar_new();
		gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR(barre), 0.1);
		gtk_box_pack_start_defaults(GTK_BOX(box), barre);
		idle=g_timeout_add(30, (GSourceFunc) gtk_progress_bar_pulse, barre);
		
		// Affichage de la fenêtre d'attente
		gtk_widget_show_all(window);
		while (gtk_events_pending ()) gtk_main_iteration ();
	
		return window;
	}
	else
	{
		g_source_remove(idle);
		idle=0;
		gtk_widget_destroy(window);
		return NULL;
	}
}

void nonblock (gint fd)		// non blocking pipes function 
{
  if (fcntl (fd, F_SETFL, O_NONBLOCK))
    {
      fprintf (stderr, "fcntl(non-blocking): %m");
      gtk_main_quit ();
    }
}

gboolean lecture_gnuchess_coup ()
{
  GtkTextIter iter_start;
  GtkTextIter iter_end;
  gchar *text;
  gchar *coup;
  static gchar compteur = 0;

  struct struct_piece_a_bouger
  {
    type_piece piece;
    gint case_arrivee;
  };
  struct struct_piece_a_bouger piece_a_bouger;
	GtkWidget *Zone_dessin;
	Zone_dessin=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "Zone dessin"));
	GtkWidget *menu_gnuchess_show_play;
	menu_gnuchess_show_play=(GtkWidget*)
						(gtk_data_list_get_by_name(widget_list, "menu gnuchess show play"));
	GtkWidget *menu_jouer_gnuchess;
	menu_jouer_gnuchess=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "menu jouer gnuchess"));
	GtkWidget *view;
	view=(GtkWidget*) (gtk_data_list_get_by_name(widget_list, "view"));
	GtkWidget *barre_outils;
	barre_outils=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "barre outils"));

  // Récupération de l'avant dernière ligne inscrite dans le textview
  while (gtk_events_pending ())
    gtk_main_iteration ();
  gtk_text_buffer_get_iter_at_line (gtk_text_view_get_buffer
				    (GTK_TEXT_VIEW (view)), &iter_start,
				    gtk_text_buffer_get_line_count
				    (gtk_text_view_get_buffer(GTK_TEXT_VIEW (view))) - 2);
  gtk_text_buffer_get_end_iter (gtk_text_view_get_buffer
																						(GTK_TEXT_VIEW (view)), &iter_end);
  text=gtk_text_buffer_get_text(gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)),
			      																		&iter_start, &iter_end, FALSE);

  if (strstr (text, "is :") && compteur < 1)
    {
      coup = g_strndup (strstr (text, "is : ") + 5,
				strstr(text,"\n") - (strstr (text, "is : ") + 5));

      traduction_insertion_coup (coup, &piece_a_bouger);
      compteur++;
      if (humain_joue == noire)
				humain_joue = blanc;
      else
				humain_joue = noire;
      g_free (coup);
      gtk_widget_queue_draw (Zone_dessin);

      while (gtk_events_pending ()) gtk_main_iteration ();
      if (gtk_check_menu_item_get_active
	  			(GTK_CHECK_MENU_ITEM (menu_gnuchess_show_play)))
			{
				GdkColormap *colormap;	//colormap de l'échiquier
				GdkColor couleur_texte;
		
				colormap = gdk_drawable_get_colormap (Zone_dessin->window);
		
				couleur_texte.red = 0;
				couleur_texte.green = 65535;
				couleur_texte.blue = 0;
		
				gdk_colormap_alloc_color (colormap, &couleur_texte, FALSE, FALSE);
				gdk_gc_set_foreground (Zone_dessin->style->
						 fg_gc[GTK_WIDGET_STATE (Zone_dessin)], &couleur_texte);
				if (g_echiquier_inverse)
				{
					gdk_draw_line (Zone_dessin->window,
						 Zone_dessin->style->
						 fg_gc[GTK_WIDGET_STATE (Zone_dessin)],
						 (gint)(TAILLE_PIECE/2)+ 20 + ((TAILLE_PIECE * 7) -
						 (echiquier[piece_a_bouger.piece.place].x -
							1) * TAILLE_PIECE),
						 (gint)(TAILLE_PIECE/2)+ 20 +
						 ((echiquier[piece_a_bouger.piece.place].y -
							 1) * TAILLE_PIECE),
						 (gint)(TAILLE_PIECE/2)+ 20 + ((TAILLE_PIECE * 7) -
						 (echiquier[piece_a_bouger.case_arrivee].x -
							1) * TAILLE_PIECE),
						 (gint)(TAILLE_PIECE/2)+ 20 +
						 ((echiquier[piece_a_bouger.case_arrivee].y -
							 1) * TAILLE_PIECE));
				}
				else
				{
					gdk_draw_line (Zone_dessin->window,
						Zone_dessin->style->
						fg_gc[GTK_WIDGET_STATE (Zone_dessin)],
						(gint)(TAILLE_PIECE/2)+ 20 +
						((echiquier[piece_a_bouger.piece.place].x - 1)* TAILLE_PIECE),
						(gint)(TAILLE_PIECE/2)+ 20 + ((TAILLE_PIECE * 7) -
						(echiquier[piece_a_bouger.piece.place].y - 1) * TAILLE_PIECE),
						(gint)(TAILLE_PIECE/2)+ 20 +
						((echiquier[piece_a_bouger.case_arrivee].x - 1)*TAILLE_PIECE),
						(gint)(TAILLE_PIECE/2)+ 20 + ((TAILLE_PIECE * 7) - 
						(echiquier[piece_a_bouger.case_arrivee].y - 1) * TAILLE_PIECE));
				}
				couleur_texte.red = 0;
				couleur_texte.green = 0;
				couleur_texte.blue = 0;
				gdk_colormap_alloc_color (colormap, &couleur_texte, FALSE, FALSE);
				gdk_gc_set_foreground (Zone_dessin->style->
												fg_gc[GTK_WIDGET_STATE (Zone_dessin)], &couleur_texte);
			}
      gtk_widget_set_sensitive (menu_jouer_gnuchess, TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET(gtk_toolbar_get_nth_item
																			(GTK_TOOLBAR (barre_outils), 6)), TRUE);
    }
  else
    compteur = 0;

  if (strstr (text, "quitter"))
    {
      g_free (text);
      gtk_text_buffer_set_text (gtk_text_view_get_buffer
				(GTK_TEXT_VIEW (view)),
				"Linechec\nDéconnecté du serveur de jeu.\n",
				-1);
      return FALSE;
    }
  else
    {
      g_free (text);
      return TRUE;
    }
}

gboolean readsocket (GIOChannel * giochan, GIOCondition * giocond,
																																	gpointer data)
{
  gchar tmp_buffer[1000];
  gint fd;
  gchar rd;

  fd = g_io_channel_unix_get_fd (giochan);
  nonblock (fd);		// fonction rendant le pipe non bloquant

  memset (tmp_buffer, '\0', 1000);
  rd = read (fd, tmp_buffer, 1000);
  insertion_texte_fen_etat_connexion (tmp_buffer);

  return lecture_gnuchess_coup ();
}

gboolean writesocket (GIOChannel * giochan, GIOCondition * giocond,
																																	gpointer data)
{
  gchar *commande = (gchar *) data;
  gint fd;
  GtkTextIter iter;
  GtkTextIter iter_start;
  GtkTextIter iter_end;
  gchar *text;
  gchar compteur = 0;
	gint i;
	GtkWidget *view;
	view=(GtkWidget*) (gtk_data_list_get_by_name(widget_list, "view"));

  fd = g_io_channel_unix_get_fd (giochan);
  nonblock (fd);		// fonction rendant le pipe non bloquant

  // Compte du nombre de commandes passé en paramêtre
  text = commande;
  if (!strstr (commande, "\n"))
    compteur = 1;
  else
    while (strstr (text, "\n"))
      {
	text = strstr (text, "\n") + 1;
	compteur++;
      }

  for (i = 0; i < compteur; i++)
    {
      do
	{
	  while (gtk_events_pending ())
	    gtk_main_iteration ();
	  gtk_text_buffer_get_start_iter (gtk_text_view_get_buffer
					  (GTK_TEXT_VIEW (view)), &iter);
	}
      while (!gtk_text_iter_forward_search
	     (&iter, "White", GTK_TEXT_SEARCH_TEXT_ONLY, &iter_start,
	      &iter_end, NULL)
	     && !gtk_text_iter_forward_search (&iter, "Black",
					       GTK_TEXT_SEARCH_TEXT_ONLY,
					       &iter_start, &iter_end, NULL));
      text =
	gtk_text_buffer_get_text (gtk_text_view_get_buffer
				  (GTK_TEXT_VIEW (view)), &iter_start,
				  &iter_end, FALSE);

      if (compteur != 1)
	text = g_strndup (commande, strstr (commande, "\n") - commande + 1);
      else
	text = g_strdup (commande);

      write (fd, text, strlen (text));
      insertion_texte_fen_etat_connexion (text);
      if (!strstr (commande, "\n") && compteur == 1)
	{
	  write (fd, "\n", 1);
	  insertion_texte_fen_etat_connexion ("\n");
	}
      commande = strstr (commande, "\n") + 1;
      while (gtk_events_pending ()) gtk_main_iteration ();
    }
  g_free (text);

  return FALSE;
}

void connexion_gnuchess ()
{
  static gint entree, sortie;
  static GPid child_pid;
  GtkTooltips *connexion_tips;	// tooltip pour le widget connexion
  const gchar *encode;		// Type d'encodage des caractères utilisé en local
	GError *error=NULL;
	static GIOChannel *channel_read=NULL;
	static GIOChannel *channel_write=NULL;

	GtkWidget *fenetre_principale;
	fenetre_principale=(GtkWidget*)
							(gtk_data_list_get_by_name(widget_list, "fenetre_principale"));
	GtkWidget *barre_outils;
	barre_outils=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "barre outils"));
	GtkWidget *Zone_dessin;
	Zone_dessin=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "Zone dessin"));
	GtkWidget *menu_nouveau;
	menu_nouveau=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "menu nouveau"));
	GtkWidget *menu_charger;
	menu_charger=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "menu charger"));
	GtkWidget *menu_jouer_gnuchess;
	menu_jouer_gnuchess=(GtkWidget*)
								(gtk_data_list_get_by_name(widget_list, "menu jouer gnuchess"));
	GtkWidget *connexion;
	connexion=(GtkWidget*) (gtk_data_list_get_by_name(widget_list, "connexion"));

  if (partie_en_cours == TRUE && partie_enregistree == FALSE)
    g_enreg_partie (fenetre_principale, NOUVELLE_PARTIE);

	// Si gnuchess est en cours d'utilisation alors on l'arrête.
	if (strstr(gtk_label_get_text(GTK_LABEL(GTK_BIN (menu_jouer_gnuchess)->child))
																													,"Arrêter gnuchess"))
	{
		GtkWidget *wait_window=NULL;

		// Création d'une fenêtre d'attente le temps que gnuchess se lance
		wait_window=g_fenetre_attente(wait_window,"Arrêt de gnuchess",TRUE);

		g_io_add_watch ((GIOChannel *) channel_write, G_IO_OUT, (GIOFunc) writesocket, (gpointer) "quit\n");
		while (gtk_events_pending ()) gtk_main_iteration ();
		
		gnuchess = FALSE;
		gtk_image_set_from_stock (GTK_IMAGE (connexion), GTK_STOCK_DISCONNECT,
		GTK_ICON_SIZE_MENU);
		connexion_tips = gtk_tooltips_new ();
		gtk_tooltips_set_tip (GTK_TOOLTIPS (connexion_tips),
									gtk_widget_get_ancestor (connexion, GTK_TYPE_EVENT_BOX),
									"Moteur de jeu déconnecté", "Moteur de jeu déconnecté");
		gtk_menu_item_label_set_label (menu_jouer_gnuchess,
				 "Jouer contre gnuchess");
		// Activation de l'icône de chargement dans le menu et le toolbar
		gtk_widget_set_sensitive (GTK_WIDGET(gtk_toolbar_get_nth_item
																		(GTK_TOOLBAR (barre_outils), 2)), TRUE);
		gtk_widget_set_sensitive (menu_charger, TRUE);
		// Activation de l'icône "nouvelle partie" dans le menu et le toolbar
		gtk_widget_set_sensitive (GTK_WIDGET(gtk_toolbar_get_nth_item
																		(GTK_TOOLBAR (barre_outils), 1)), TRUE);
		gtk_widget_set_sensitive (menu_nouveau, TRUE);
		// Destruction de l'iochannel
		g_io_channel_shutdown((GIOChannel *) channel_write,TRUE,&error);
		if (error)
		{
			g_print("%s\n",error->message);
			g_error_free(error);
		}
		error=NULL;
		g_io_channel_shutdown((GIOChannel *) (GIOChannel *) channel_read,TRUE,&error);
		if (error)
		{
			printf("%s\n",error->message);
			g_error_free(error);
		}
		// Destruction de la fenêtre d'attente
		g_fenetre_attente(wait_window,NULL,FALSE);
	}
	else
	{
		gchar *tmp;
		GtkWidget *wait_window=NULL;

		// Création d'une fenêtre d'attente le temps que gnuchess se lance
		wait_window=g_fenetre_attente(wait_window,"Lancement de gnuchess",TRUE);

		// Echiquier insensible tant que gnuchess n'est pas correctement initialisé
		gtk_widget_set_sensitive (Zone_dessin, FALSE);

		tmp=g_strdup("gnuchess");
		if (g_spawn_async_with_pipes(NULL, &tmp, NULL,
											G_SPAWN_SEARCH_PATH|G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
											NULL, NULL, &child_pid, &entree, &sortie, NULL, NULL))
		{
			// Initialisation de l'échiquier
			g_initialisation_jeu ();
			
			gnuchess = TRUE;
			gtk_image_set_from_stock (GTK_IMAGE (connexion), GTK_STOCK_CONNECT,
			GTK_ICON_SIZE_MENU);
			connexion_tips = gtk_tooltips_new ();
			gtk_tooltips_set_tip (GTK_TOOLTIPS (connexion_tips),
			gtk_widget_get_ancestor (connexion, GTK_TYPE_EVENT_BOX),
			"Moteur de jeu connecté", "Moteur de jeu connecté");
			gtk_label_set_label (GTK_LABEL(GTK_BIN (menu_jouer_gnuchess)->child),
					"Arrêter gnuchess");
			// Désactivation de l'icône de chargement dans le menu et le toolbar
			gtk_widget_set_sensitive (GTK_WIDGET(gtk_toolbar_get_nth_item
																			(GTK_TOOLBAR (barre_outils), 2)), FALSE);
			gtk_widget_set_sensitive (menu_charger, FALSE);
			// Désactivation de l'icône "nouvelle partie" dans le menu et le toolbar
			gtk_widget_set_sensitive (GTK_WIDGET(gtk_toolbar_get_nth_item
																			(GTK_TOOLBAR (barre_outils), 1)), FALSE);
			gtk_widget_set_sensitive (menu_nouveau, FALSE);

			channel_read = g_io_channel_unix_new (sortie);
			
			if (!g_io_channel_get_buffered (channel_read))
			{
				g_io_channel_set_buffered (channel_read, FALSE);
			}
			insertion_texte_fen_etat_connexion ("Encodage interne : ");
			insertion_texte_fen_etat_connexion (g_io_channel_get_encoding
			((GIOChannel *) channel_read));
			insertion_texte_fen_etat_connexion ("\nEncodage local : ");
			if (!g_get_charset (&encode))	// Si l'encodage local n'est pas UTF8 alors
			{
				insertion_texte_fen_etat_connexion (encode);
				insertion_texte_fen_etat_connexion ("\n");
				g_io_channel_set_encoding ((GIOChannel *) channel_read, encode,
				NULL);
			}
			g_io_add_watch ((GIOChannel *) channel_read, G_IO_IN, (GIOFunc) readsocket, NULL);
			
			while (gtk_events_pending ()) gtk_main_iteration ();
			channel_write = g_io_channel_unix_new (entree);
			widget_list=gtk_data_list_add_by_name(widget_list, channel_write,
																															"channel write");
			if (!g_io_channel_get_buffered (channel_write))
			{
				g_io_channel_set_buffered (channel_write, FALSE);
			}
			
			if (!g_get_charset (&encode))	// Si l'encodage local n'est pas UTF8 alors
			{
				g_io_channel_set_encoding (channel_write, encode, NULL);
			}
			g_io_add_watch ((GIOChannel *) channel_write, G_IO_OUT, (GIOFunc) writesocket, (gpointer) "protover 2\nnew\nrandom\n");
			// gnuchess est maintenant initialisé. On active l'échiquier.
			gtk_widget_set_sensitive (Zone_dessin, TRUE);
		}
		else
			insertion_texte_fen_etat_connexion("Le lancement de gnuchess à échoué!\n");
		// Destruction de la fenêtre d'attente
		g_fenetre_attente(wait_window,NULL,FALSE);
		g_free (tmp);
	}
}
